{% load aa_forum %}

<script
    src="{% aa_forum_static 'aa_forum/javascript/aa-forum-desktop-widgets.min.js' %}"
    integrity="sha512-j9Ul8GQz4yPBHeejEuDQBq+7OLWcuTnO5p/QcMfI8DEXLVYx2Gbx8WDWBWJ3ys40IM7k9toLsBYC4UPKylRsYw=="
    crossorigin="anonymous"
></script>
