{% load aa_forum %}

<script
    src="{% aa_forum_static 'aa_forum/javascript/aa-forum-bootstrap.min.js' %}"
    integrity="sha512-TIDI8ED1NX4aXALuB8OEfPpRtL1C1RaX09YPz9kk75iM15pDvt056463xKNHVsJ+CAi58vbv3EuGU8VxKvXQaQ=="
    crossorigin="anonymous"
></script>
