{% if perms.aa_forum.basic_access %}
    <div id="aa-forum-dashboard-widgets" class="aa-forum col-12 mb-3 collapse">
        {% if user_profile.show_unread_topics_dashboard_widget %}
            <div id="aa-forum-dashboard-widget-unread-topics" class="col-12 align-self-stretch collapse">
                <div class="aa-forum-dashboard-widget-unread-topics-content"></div>
            </div>
        {% endif %}

        <div class="aa-forum-svg-wrapper">
            {% include "aa_forum/bundles/svg/aa-forum-icons.svg" %}
        </div>

        <script>
            const aaForumDashboardWidgetsSettings = {
                unreadTopicsWidget: {
                    ajaxUrl: '{% url "aa_forum:widgets_ajax_unread_topics" %}',
                },
            };
        </script>

        {% include "aa_forum/bundles/aa-forum-css.html" %}
        {% include "aa_forum/bundles/aa-forum-dashboard-widgets-js.html" %}
    </div>
{% endif %}
