{% load aa_forum %}

<script
    src="{% aa_forum_static 'aa_forum/javascript/aa-forum-oembed.min.js' %}"
    integrity="sha512-/Dbm1O92EWo/GHfamwW3bT5Da8M6y4Fdp4xn/79D8mW2h9G+zi7u0TMTpdOx+HYzb5VrunmnLsNSg3bZHxxY9g=="
    crossorigin="anonymous"
></script>
