# Generated by Django 3.2.7 on 2021-09-10 13:37

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("aa_forum", "0002_default_settings"),
    ]

    operations = [
        migrations.AddField(
            model_name="board",
            name="discord_webhook",
            field=models.CharField(blank=True, default=None, max_length=254, null=True),
        ),
    ]
