{% load aa_forum %}

<script
    src="{% aa_forum_static 'aa_forum/javascript/aa-forum-ckeditor.min.js' %}"
    integrity="sha512-JlA2mPDhvTVOELM66o/pzh2eHi3Y9FzEfD1FM+Qe9JcsJLnmqLgH+tX8RNOAx/6JR328eBeTRE3twEVDlXcsAg=="
    crossorigin="anonymous"
></script>
