{% load aa_forum %}

<script
    src="{% aa_forum_static 'aa_forum/javascript/aa-forum-admin.min.js' %}"
    integrity="sha512-qtex/VOPoM2/H8+k9MMhpo9/DYUsTK1Qr3PMMOH5IiET710Wp2p7dh9RqTq4bVbcmhGYyEZHCytoZpAncfxB3w=="
    crossorigin="anonymous"
></script>
