{% load static %}

<script
    src="{% static 'aa_forum/libs/sumoselect/3.4.8/jquery.sumoselect.min.js' %}"
    integrity="sha512-auSlGnwkJFs/mb8KzTn/5rtvj6kjD4+llRLLgF5UxODiiHreETjShA93o51yDWhC0qR09QcALePfHXZfbRyX7Q=="
    crossorigin="anonymous"
></script>
