{% load i18n %}

<div class="card">
    <div class="card-body aa-forum-unread-topics-loop">
        {% translate "Forum: Unread topics" as widget_title %}
        {% include "framework/dashboard/widget-title.html" with title=widget_title %}

        {% for board in boards %}
            {% include "aa_forum/partials/forum/topic/unread-topics.html" %}
        {% endfor %}
    </div>
</div>
