{% load static %}

<link
    rel="stylesheet"
    href="{% static 'aa_forum/libs/sumoselect/3.4.8/sumoselect.min.css' %}"
    integrity="sha512-vU7JgiHMfDcQR9wyT/Ye0EAAPJDHchJrouBpS9gfnq3vs4UGGE++HNL3laUYQCoxGLboeFD+EwbZafw7tbsLvg=="
    crossorigin="anonymous"
>
