{% load aa_forum %}

<script
    src="{% aa_forum_static 'aa_forum/javascript/aa-forum-personal-messages.min.js' %}"
    integrity="sha512-usgrVEyJhRRGkR6SSjt07jZ7dYlSdb0zahsfmK3ZuBZkUq2CuxkkcpZvvoA5XmxturaAYtuRXIJ8Epq+92qFdA=="
    crossorigin="anonymous"
></script>
