# Generated by Django 4.0.3 on 2022-04-08 18:44

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("aa_forum", "0004_board_use_webhook_for_replies"),
    ]

    operations = [
        migrations.AddField(
            model_name="board",
            name="announcement_groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="User in at least one of the selected groups will be able to start topics in 'Announcement Boards'. If no group is selected, only forum admins can do so.",
                related_name="aa_forum_boards_group_start_topic_restriction",
                to="auth.group",
            ),
        ),
        migrations.AddField(
            model_name="board",
            name="is_announcement_board",
            field=models.BooleanField(
                default=False,
                help_text="Mark this board as an 'Announcement Board', meaning that only certain selected groups can start new topics. All others who have access to this board will still be able to discuss in the topics though. (Default: NO)",
            ),
        ),
        migrations.AlterField(
            model_name="board",
            name="groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="User in at least one of these groups have access to this board. If no groups are selected, everyone with access to the forum has also access to this board.",
                related_name="aa_forum_boards_group_restriction",
                to="auth.group",
            ),
        ),
    ]
