{% extends "aa_forum/base.html" %}

{% load i18n %}

{% block page_title %}
    {% translate "Modify message" as page_title %}
    {% translate "Forum" as app_title %}

    {{ page_title|title }} » {{ app_title }}
{% endblock %}

{% block aa_forum_header %}
    {% include "aa_forum/partials/breadcrumb.html" with category=board.category board=board topic=message.topic modify_message=True %}
{% endblock %}

{% block aa_forum_body %}
    {% include "aa_forum/partials/forum/topic/modify-message.html" %}
{% endblock %}

{% block extra_css %}
    {% include "aa_forum/bundles/ckeditor5-css.html" %}
    {% include "aa_forum/bundles/aa-forum-css.html" %}
{% endblock %}

{% block extra_javascript %}
    {% include "aa_forum/bundles/ckeditor5-js.html" %}
{% endblock %}
